
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS
/* Enumerations from "/build/endeavour/src/build/../Endeavour/src/animation/gtd-animation-enums.h" */
#define GTD_TYPE_EASE_MODE	(gtd_ease_mode_get_type())
GType gtd_ease_mode_get_type	(void) G_GNUC_CONST;
#define GTD_TYPE_TIMELINE_DIRECTION	(gtd_timeline_direction_get_type())
GType gtd_timeline_direction_get_type	(void) G_GNUC_CONST;
G_END_DECLS

/* Generated data ends here */

